@@ -1,16 +1,25 @@
 
 
 
+import typing
+
+
+
 from .ArgsOptionDataDict import ArgsOptionDataDict
 
 
 
 
+
 #
 # This class contains the results of command line parsing.
 #
 class ParsedArgs(object):
 
+	################################################################################################################################
+	## Constructor
+	################################################################################################################################
+
 	def __init__(self, commands):
 		self.__commands = commands
 		self.__optionData = ArgsOptionDataDict()
@@ -20,7 +29,9 @@ class ParsedArgs(object):
 		self.__bError = False					# NEW IMPL
 	#
 
-
+	################################################################################################################################
+	## Public Properties
+	################################################################################################################################
 
 	#
 	# This dictionary is ready to store all data parsed from processing command
@@ -31,7 +42,9 @@ class ParsedArgs(object):
 		return self.__optionData
 	#
 
-
+	################################################################################################################################
+	## Public Methods
+	################################################################################################################################
 
 	def dump(self, prefix:str = None, printFunction = None):
 		if prefix is None:
@@ -51,8 +64,6 @@ class ParsedArgs(object):
 		printFunction(prefix + "]")
 	#
 
-
-
 	def parseNextCommand(self):
 		if self.__bError:															# NEW IMPL
 			raise Exception("There have been previous parsing errors!")				# NEW IMPL
@@ -79,8 +90,6 @@ class ParsedArgs(object):
 		return (cmd.name, optionArgs)
 	#
 
-
-
 	def parseCommands(self):														# NEW IMPL
 		if self.__bError:
 			raise Exception("There have been previous parsing errors!")
@@ -105,8 +114,6 @@ class ParsedArgs(object):
 			yield (cmd.name, optionArgs)
 	#
 
-
-
 #
 
 
