@@ -1,8 +1,13 @@
-﻿from .ArgOption import ArgOption
+﻿
+
+
+__version__ = "0.2021.3.7"
+
+
+
+from .ArgOption import ArgOption
 from .ArgsParser import ArgsParser
 from .ParsedArgs import ParsedArgs
 
 
 
-
-__version__ = "0.2021.1.18"
\ No newline at end of file
